import logging
from inspect import isfunction


logger = logging.getLogger(__name__)


class Tag(object):
    """Class that abstract tags."""
    def __init__(self, name, valuable, ttl):
        """
        :param string name: tag name
        :param valuable: something that gives tag value, string or function
        :param None,int ttl: Time to live for caching the tags
        """
        self.name = name
        self._value = valuable
        self.ttl = ttl

    @property
    def value(self):
        """Returns tag value."""
        if isfunction(self._value):
            try:
                return self._value()
            except Exception:
                logger.exception(u'Calculating tag %s failed:' % self.name)
                return None

        return self._value


class DynamicTags(object):
    """Set tags dynamicaly."""
    def __init__(self):
        """Here define the tags."""
        self.tags = []

    def iter_tags(self):
        for t in self.tags:
            yield t
